/*
 * ------------------------------------------------------------------------
 *     NAME: tree - integrating C code into the Tree class
 *
 * DESCRIPTION:
 *   This example shows how Tcl-style command handlers can be
 *   integrated into [incr Tcl] classes.  In this example, the
 *   "add" method for the Tree class is implemented with C code.
 * ------------------------------------------------------------------------
 *     BOOK: Tcl/Tk Extensions for O'Reilly & Associates
 *   AUTHOR: Michael J. McLennan, AT&T Bell Laboratories
 * ========================================================================
 *               Copyright (c) 1996  AT&T Bell Laboratories
 * ========================================================================
 */
#include <tcl.h>


/*
 * ------------------------------------------------------------------------
 *  Tree_AddCmd()
 *
 *  Invoked by Tcl to handle the "add" method in the Tree class.
 *  Takes the name of a Tree object and adds it as a child of
 *  the current node.  Handles the following syntax:
 *
 *    add <treeObj>
 *
 * ------------------------------------------------------------------------
 */
int
Tree_AddCmd(dummy, interp, argc, argv)
    ClientData dummy;        /* unused */
    Tcl_Interp *interp;      /* current interpreter */
    int argc;                /* number of arguments */
    char **argv;             /* argument strings */
{
    char *val;
    Tcl_DString buffer;

    if (argc != 2) {
        Tcl_AppendResult(interp, "wrong # args: should be \"",
            argv[0], " treeObj\"", (char*)NULL);
        return TCL_ERROR;
    }

    /*
     *  Build a command string like "treeObj parent $this" and
     *  execute it.
     */
    Tcl_DStringInit(&buffer);
    val = Tcl_GetVar(interp, "this", TCL_LEAVE_ERR_MSG);
    if (val == NULL) {
        Tcl_DStringFree(&buffer);
        return TCL_ERROR;
    }
    Tcl_DStringAppendElement(&buffer, argv[1]);
    Tcl_DStringAppendElement(&buffer, "parent");
    Tcl_DStringAppendElement(&buffer, val);
    val = Tcl_DStringValue(&buffer);

    if (Tcl_Eval(interp,val) != TCL_OK) {
        Tcl_DStringFree(&buffer);
        return TCL_ERROR;
    }
    Tcl_ResetResult(interp);

    /*
     *  Add the specified object to the "children" list.
     */
    val = Tcl_SetVar(interp, "children", argv[1],
        TCL_LEAVE_ERR_MSG | TCL_LIST_ELEMENT | TCL_APPEND_VALUE);

    if (val == NULL) {
        Tcl_DStringFree(&buffer);
        return TCL_ERROR;
    }

    Tcl_DStringFree(&buffer);
    return TCL_OK;
}
